@@ -1,4 +1,26 @@
 #!/bin/sh
+#
+# SPDX-FileCopyrightText: Copyright (c) 2004-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+# SPDX-License-Identifier: MIT
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
+#
 
 PATH="/sbin:/usr/sbin:$PATH"
 
@@ -291,6 +313,64 @@ check_command() {
     return 0
 }
 
+# Function to discover available IB devices
+discover_ib_devices() {
+    ib_devices=""
+    fallback_devices=""
+    
+    # Check if /sys/class/infiniband directory exists
+    if [ ! -d "/sys/class/infiniband" ]; then
+        return 1
+    fi
+    
+    # Loop through each InfiniBand device directory
+    for dir in /sys/class/infiniband/*/device; do
+        # Check if the device directory exists
+        if [ -d "$dir" ]; then
+            # Define the path to the VPD file
+            vpd_file="$dir/vpd"
+            
+            # Check if the VPD file exists and contains SW_MNG (NVL5+ system)
+            if [ -f "$vpd_file" ] && grep -q "SW_MNG" "$vpd_file" 2>/dev/null; then
+                # Extract the InfiniBand device name using parameter expansion
+                device_name="${dir%/device}"  # Removes '/device' from the end of $dir
+                device_name="${device_name##*/}"  # Extracts the part after the last '/'
+                
+                # Add to fallback list (all SW_MNG devices with non-empty names)
+                if [ -n "$device_name" ]; then
+                    fallback_devices="$fallback_devices $device_name"
+                    
+                    # Check if pkey file exists for this device
+                    pkey_file="/sys/class/infiniband/$device_name/ports/1/pkeys/0"
+                    if [ -f "$pkey_file" ]; then
+                        pkey_value=$(cat "$pkey_file" 2>/dev/null)
+                        # Only include devices that are full members (0xffff) in primary list
+                        if [ "$pkey_value" = "0xffff" ]; then
+                            ib_devices="$ib_devices $device_name"
+                        fi
+                    fi
+                fi
+            fi
+        fi
+    done
+    
+    # Remove leading spaces
+    ib_devices=$(echo "$ib_devices" | sed 's/^[[:space:]]*//')
+    fallback_devices=$(echo "$fallback_devices" | sed 's/^[[:space:]]*//')
+    
+    # If no full member devices found, fall back to all SW_MNG devices
+    if [ -z "$ib_devices" ] && [ -n "$fallback_devices" ]; then
+        ib_devices="$fallback_devices"
+    fi
+    
+    # If still no devices, fall back to default mlx5_0 if it exists
+    if [ -z "$ib_devices" ] && [ -d "/sys/class/infiniband/mlx5_0" ]; then
+        ib_devices="mlx5_0"
+    fi
+    
+    echo "$ib_devices"
+}
+
 run_query_mlxlink() {
     description="$1"  # e.g., "Mellanox Link Firmware Versions"
     query="$2"        # Query string to match in 'mst status'
@@ -299,6 +379,13 @@ run_query_mlxlink() {
     if ! check_command "mst" "$description"; then return; fi
     if ! check_command "mlxlink" "$description"; then return; fi
 
+    # Discover available IB devices
+    ib_devices=$(discover_ib_devices)
+    if [ -z "$ib_devices" ]; then
+        store_result "$description" "No IB devices found"
+        return
+    fi
+
     # Run mst status and capture output
     mst_output=$(sudo mst status -v 2>&1)
     if [ $? -ne 0 ]; then
@@ -308,31 +395,32 @@ run_query_mlxlink() {
 
     # Initialize result aggregation
     aggregated_output=""
-    while IFS= read -r tmp_line; do
-        echo "$tmp_line" | grep "$query" | grep "mlx5_0" >/dev/null 2>&1
-        if [ $? -eq 0 ]; then
-            # Extract device name
-            devName=$(echo "$tmp_line" | awk -F" " '{print $1}')
-            mlxlink_output=$(sudo mlxlink -d "$devName" 2>&1)
+    
+    # Process each discovered IB device
+    for ib_device in $ib_devices; do
+        while IFS= read -r tmp_line; do
+            echo "$tmp_line" | grep "$query" | grep "$ib_device" >/dev/null 2>&1
+            if [ $? -eq 0 ]; then
+                # Extract device name
+                devName=$(echo "$tmp_line" | awk -F" " '{print $1}')
+                mlxlink_output=$(sudo mlxlink -d "$devName" 2>&1)
 
-            if [ $? -ne 0 ]; then
-                aggregated_output="$aggregated_output""Device: $devName
+                if [ $? -ne 0 ]; then
+                    aggregated_output="$aggregated_output""Device: $devName
 Error: $(echo "$mlxlink_output" | head -n 1)
 "
-                continue
-            fi
+                    continue
+                fi
 
-            aggregated_output="$aggregated_output""
-""
-""Device: $devName
+                aggregated_output="$aggregated_output""Device: $devName
 $mlxlink_output
 
-
 "
-        fi
-    done <<EOF
-$(echo "$mst_output" | grep "$query" | grep "mlx5_0")
+            fi
+        done <<EOF
+$(echo "$mst_output" | grep "$query" | grep "$ib_device")
 EOF
+    done
 
     # Store the result
     if [ -z "$aggregated_output" ]; then
@@ -528,7 +616,7 @@ usage() {
     echo ""
 }
 
-NVIDIA_BUG_REPORT_CHANGE='$Change: 36405693 $'
+NVIDIA_BUG_REPORT_CHANGE='$Change: 36557123 $'
 NVIDIA_BUG_REPORT_VERSION=`echo "$NVIDIA_BUG_REPORT_CHANGE" | tr -c -d "[:digit:]"`
 
 # Set the default filename so that it won't be empty in the usage message
@@ -1584,6 +1672,7 @@ append "/proc/version"
 append "/proc/pci"
 append "/proc/iomem"
 append "/proc/mtrr"
+append "/proc/buddyinfo"
 
 append "/proc/driver/nvidia/version"
 append "/proc/driver/nvidia/params"
@@ -1594,6 +1683,7 @@ if [ $BUG_REPORT_SAFE_MODE -eq 0 ]; then
         append "/proc/driver/nvidia/gpus/$GPU/information"
     done
     append_glob "/proc/driver/nvidia/warnings/*"
+    append_glob "/proc/driver/nvidia-modeset/*"
 fi
 
 append_glob "/proc/acpi/video/*/*/info"
@@ -1979,17 +2069,25 @@ dump_mlx() {
             # loads PCI driver & creates PCI devices file descriptors under /dev/mst/.
             echo "____________________________________________"
             echo ""
-            report_command "$mst start"
+            report_command "timeout 60 $mst start"
             echo "____________________________________________"
             echo ""
 
             # scans and adds gpu devices
-            report_command "$mst gpu add"
+            report_command "timeout 60 $mst gpu add"
             echo "____________________________________________"
             echo ""
 
-            # scans and add ib devices
-            report_command "$mst ib add mlx5_0"
+            # Discover and add IB devices dynamically
+            ib_devices=$(discover_ib_devices)
+            if [ -n "$ib_devices" ]; then
+                for ib_device in $ib_devices; do
+                    report_command "timeout 60 $mst ib add $ib_device"
+                done
+            else
+                # Fallback to mlx5_0 if no devices discovered
+                report_command "timeout 60 $mst ib add mlx5_0"
+            fi
             echo "____________________________________________"
             echo ""
 
@@ -1998,13 +2096,15 @@ dump_mlx() {
             mlxreg=`which mlxreg 2> /dev/null | head -n 1`
 
             # dumps all the scanned devices
-            report_command "$mst status -v"
+            report_command "timeout 60 $mst status -v"
             echo "____________________________________________"
             echo ""
 
             # Iterate and get dumps for each of the supported devices
             "$mst" status -v |
             {
+                # Get IB devices for use in subshell
+                ib_devices_shell="$ib_devices"
                 while read tmp_line
                 do
                 echo $tmp_line | grep "netir" > /dev/null  2>&1
@@ -2021,41 +2121,50 @@ dump_mlx() {
                         fi
                     fi
                 fi
-                echo $tmp_line | grep "Quantum" | grep "mlx5_0" > /dev/null 2>&1
-                if [ $? -eq 0 ]; then
-                    devName=`echo $tmp_line | awk -F" " 'NR == 1 {print $1}';`
-                    if [ ! -z "$devName" ]; then
-                        echo "Starting Quantum dump..."$tmp_line
-                        echo "____________________________________________"
-                        dump_mlx "$mlxlink" "$devName"
-                        if [ ! -z "$resourcedump" ]; then
-                            report_command "$resourcedump dump -d $devName -s 0x5024"
+                # Check for Quantum devices on any discovered IB device
+                for ib_device in $ib_devices_shell; do
+                    echo $tmp_line | grep "Quantum" | grep "$ib_device" > /dev/null 2>&1
+                    if [ $? -eq 0 ]; then
+                        devName=`echo $tmp_line | awk -F" " 'NR == 1 {print $1}';`
+                        if [ ! -z "$devName" ]; then
+                            echo "Starting Quantum dump..."$tmp_line
                             echo "____________________________________________"
-                            echo "Starting Quantum flash dump..."
-                            echo "____________________________________________"
-                            report_command "$resourcedump dump -d $devName -s 0x5028"
-                            echo "____________________________________________"
-                            if [ ! -z "$mlxreg" ]; then
-                                echo "Clearing Quantum flash dump..."
+                            dump_mlx "$mlxlink" "$devName"
+                            if [ ! -z "$resourcedump" ]; then
+                                report_command "$resourcedump dump -d $devName -s 0x5024"
                                 echo "____________________________________________"
-                                report_command "$mlxreg -d $devName --set erase_log=0x1 --reg_name MOFDE -y"
+                                echo "Starting Quantum flash dump..."
                                 echo "____________________________________________"
+                                report_command "$resourcedump dump -d $devName -s 0x5028"
+                                echo "____________________________________________"
+                                if [ ! -z "$mlxreg" ]; then
+                                    echo "Clearing Quantum flash dump..."
+                                    echo "____________________________________________"
+                                    report_command "$mlxreg -d $devName --set erase_log=0x1 --reg_name MOFDE -y"
+                                    echo "____________________________________________"
+                                fi
+                                echo ""
                             fi
-                            echo ""
                         fi
+                        break
                     fi
-                fi
+                done
                 echo $tmp_line | grep "ConnectX" > /dev/null 2>&1
                 if [ $? -eq 0 ]; then
                     devName=`echo $tmp_line | awk -F" " 'NR == 1 {print $3}';`
                     if [ ! -z "$devName" ]; then
-                        echo "Starting ConnectX dump..."$tmp_line
-                        echo "____________________________________________"
-                        dump_mlx "$mlxlink" "$devName"
-                        if [ ! -z "$mstdump" ]; then
-                            report_command "$mstdump $devName"
+                        # Check if this is a primary function (.0) or VF (.1, .2, etc.)
+                        if echo "$devName" | grep -q "\.0$"; then
+                            echo "Starting ConnectX dump (PRIMARY FUNCTION)($devName)..."$tmp_line
                             echo "____________________________________________"
-                            echo ""
+                            dump_mlx "$mlxlink" "$devName"
+                            if [ ! -z "$mstdump" ]; then
+                                report_command "$mstdump $devName"
+                                echo "____________________________________________"
+                                echo ""
+                            fi
+                        else
+                            echo "Skipping ConnectX VF (skipping mstdump)($devName)..."$tmp_line
                         fi
                     fi
                 fi
